#ifndef __OLPC_XO_1_75_H
#define __OLPC_XO_1_75_H

struct olpc_platform_t {
	int flags;
	uint32_t boardrev;
	int ecver;
};

#define OLPC_F_PRESENT		0x01
#define OLPC_F_DCON		0x02
#define OLPC_F_NO_DCON_RAM	0x04

/*
 * for XO-1, the revision in hex is equivalent to the common name
 * for the revision -- i.e. board B3 has revision 0xb3.  with the
 * introduction of XO 1.5 boards, the EC will no longer increment
 * the first nibble of the ID, since otherwise we'll run out of
 * namespace too soon.  so we'll start mapping with macros from
 * 0xd0 onwards.
 */
#define BOARD_XO_1_B1	0xb1
#define BOARD_XO_1_B2	0xb2
#define BOARD_XO_1_B3	0xb3
#define BOARD_XO_1_C1	0xc1
#define BOARD_XO_1_C2	0xc2

#define BOARD_XO_1_5_Ax	0xd0   // A0 and A1 boards have the same ID
#define BOARD_XO_1_5_B1	0xd1
#define BOARD_XO_1_5_B2	0xd1   // B2 are also 0xd1.  all B1 were destroyed.
#define BOARD_XO_1_5_B3	0xd2
#define BOARD_XO_1_5_C1	0xd3   // initial factory build (12/2009)
#define BOARD_XO_1_5_C2	0xd4   // ramp production (2/2010)

// FIXME -- add 1.75 board ids, and move all of this platform
// stuff to a proper home.

extern struct olpc_platform_t olpc_platform_info;

/*
 * OLPC board IDs contain the major build number within the mask 0xfff0,
 * and the minor build number within 0x000f.  Pre-builds have a minor
 * number less than 8, and normal builds start at 8.  For example, 0x0B10
 * is a PreB1, and 0x0C18 is a C1.  A first digit of '0' represents XO-1
 * or XO-1.5.  A '1' represents 1.75.  So 0x1A3 is version A3 of 1.75.
 */

static inline uint32_t olpc_board(uint16_t id)
{
	return (id << 4) | 0x8;
}

static inline uint32_t olpc_board_pre(uint16_t id)
{
	return id << 4;
}

static inline int machine_is_olpc(void)
{
	return (olpc_platform_info.flags & OLPC_F_PRESENT) ? 1 : 0;
}

/*
 * The DCON is OLPC's Display Controller.  It has a number of unique
 * features that we might want to take advantage of..
 */
static inline int olpc_has_dcon(void)
{
	return (olpc_platform_info.flags & OLPC_F_DCON) ? 1 : 0;
}

static inline int olpc_missing_dcon_ram(void)
{
	return (olpc_platform_info.flags & OLPC_F_NO_DCON_RAM) ? 1 : 0;
}

/*
 * XO-1/XO-1.5
 * The "Mass Production" version of OLPC's XO is identified as being model
 * C2.  During the prototype phase, the following models (in chronological
 * order) were created: A1, B1, B2, B3, B4, C1.  The A1 through B2 models
 * were based on Geode GX CPUs, and models after that were based upon
 * Geode LX CPUs.  There were also some hand-assembled models floating
 * around, referred to as PreB1, PreB2, etc.
 *
 * XO-1.75
 * Realizing that we needed another specifier, we expanded the board-id
 * to 2 bytes, from 1.  So numbering is similar to above, with a '1'
 * in front.
 */
static inline int olpc_board_at_least(uint32_t rev)
{
	return olpc_platform_info.boardrev >= rev;
}

/*
 * Register and unregister a function to be used to submit commands
 * to the EC with.  Called by the olpc-ec-1-75 module.
 */
void olpc_xo_1_75_ec_cmd_handler_set(int (*fn)(void *cookie, u8 *cmd,
					       u8 *resp, int resp_len),
				     void *cookie);
void olpc_xo_1_75_ec_cmd_handler_clear(void);

/*
 * Submit an 8-byte command to the EC.  Returns the number of bytes
 * returned by the EC, or a negative error code.  If the EC driver is
 * not loaded, returns -EOPNOTSUPP.
 */
int olpc_xo_1_75_ec_cmd(u8 *cmd, u8 *resp, int resp_len);


/*
 * wrapper for the above routine, to support "legacy" modules
 */
int olpc_ec_cmd(unsigned char cmd, unsigned char *inbuf, size_t inlen,
		unsigned char *outbuf, size_t outlen);

/*
 * manage and receive EC event notifications
 */
void olpc_ec_disable_wakeup(int event);
void olpc_ec_register_event_callback(void (*f)(int),int);
void olpc_ec_deregister_event_callback(int);

/*
 * EC commands (from http://dev.laptop.org/git/users/rsmith/ec-1.75/tree/ec_cmd.h)
 */
#define CMD_GET_API_VERSION              0x08  // out: u8 
#define CMD_READ_VOLTAGE                 0x10  // out: u16, *9.76/32,    mV
# define EC_BAT_VOLTAGE CMD_READ_VOLTAGE                 
#define CMD_READ_CURRENT                 0x11  // out: s16, *15.625/120, mA
# define EC_BAT_CURRENT CMD_READ_CURRENT                 
#define CMD_READ_ACR                     0x12  // out: s16, *6250/15,    Ah
# define EC_BAT_ACR CMD_READ_ACR                     
#define CMD_READ_BATT_TEMPERATURE        0x13  // out: u16, *100/256,   deg C
# define EC_BAT_TEMP CMD_READ_BATT_TEMPERATURE        
#define CMD_READ_AMBIENT_TEMPERATURE     0x14  // unimplemented, no hardware
# define EC_AMB_TEMP CMD_READ_AMBIENT_TEMPERATURE     
#define CMD_READ_BATTERY_STATUS          0x15  // out: u8, bitmask
# define EC_BAT_STATUS CMD_READ_BATTERY_STATUS          
#define CMD_READ_SOC                     0x16  // out: u8, percentage
# define EC_BAT_SOC CMD_READ_SOC                     
#define CMD_READ_GAUGE_ID                0x17  // out: u8 * 8    
# define EC_BAT_SERIAL CMD_READ_GAUGE_ID                
#define CMD_READ_GAUGE_DATA              0x18  // in: u8 addr, out: u8 data 
# define EC_BAT_EEPROM CMD_READ_GAUGE_DATA              
#define CMD_READ_BOARD_ID                0x19  // out: u16 (platform id) 
#define CMD_READ_BATT_ERR_CODE           0x1f  // out: u8, error bitmask
# define EC_BAT_ERRCODE CMD_READ_BATT_ERR_CODE           
#define CMD_SET_DCON_POWER               0x26  // in: u8 
#define CMD_RESET_EC                     0x28  // none 
#define CMD_READ_BATTERY_TYPE            0x2c  // out: u8  
#define CMD_SET_AUTOWAK                  0x33  // out: u8 
#define CMD_SET_EC_WAKEUP_TIMER          0x36  // in: u32, out: ? 
#define CMD_READ_EXT_SCI_MASK            0x37  // ? 
#define CMD_WRITE_EXT_SCI_MASK           0x38  // ? 
#define CMD_CLEAR_EC_WAKEUP_TIMER        0x39  // none 
#define CMD_ENABLE_RUNIN_DISCHARGE       0x3B  // none 
#define CMD_DISABLE_RUNIN_DISCHARGE      0x3C  // none 
#define CMD_READ_MPPT_ACTIVE             0x3d  // out: u8 
#define CMD_READ_MPPT_LIMIT              0x3e  // out: u8 
#define CMD_SET_MPPT_LIMIT               0x3f  // in: u8 
#define CMD_DISABLE_MPPT                 0x40  // none 
#define CMD_ENABLE_MPPT                  0x41  // none 
#define CMD_READ_VIN                     0x42  // out: u16 
#define CMD_EXT_SCI_QUERY                0x43  // ? 
#define RSP_KEYBOARD_DATA                0x48  // ? 
#define RSP_TOUCHPAD_DATA                0x49  // ? 
#define CMD_GET_FW_VERSION               0x4a  // out: u8 * 16 
#define CMD_POWER_CYCLE                  0x4b  // none 
#define CMD_POWER_OFF                    0x4c  // none 
#define CMD_RESET_EC_SOFT                0x4d  // none 
#define CMD_READ_GAUGE_U16               0x4e  // ? 
#define CMD_ENABLE_MOUSE                 0x4f  // ? 
#define CMD_JUMP_TO_UPDATE               0x50  // (A1 only)  out: u8 
#define CMD_UPDATE_FLASH                 0x51  // (A1 only) 
#define CMD_ECHO                         0x52  // in: u8 * 5, out: u8 * 5 
#define CMD_GET_FW_DATE                  0x53  // out: u8 * 16 
#define CMD_GET_FW_USER                  0x54  // out: u8 * 16 
#define CMD_TURN_OFF_POWER               0x55  // none (same as 0x4c)
#define CMD_READ_OLS                     0x56  // out: u16 
#define CMD_OLS_SMT_LEDON                0x57  // none 
#define CMD_OLS_SMT_LEDOFF               0x58  // none 
#define CMD_START_OLS_ASSY               0x59  // none 
#define CMD_STOP_OLS_ASSY                0x5a  // none 
#define CMD_OLS_SMTTEST_STOP             0x5b  // none 
#define CMD_READ_VIN_SCALED              0x5c  // out: u16
#define CMD_READ_BAT_MIN_W               0x5d  // out: u16
#define CMD_READ_BAR_MAX_W               0x5e  // out: u16
#define CMD_RESET_BAT_MINMAX_W           0x5f  // none

#define CMD_READ_LOCATION                0x60  // in: u16 addr, out: u8 data 
#define CMD_WRITE_LOCATION               0x61  // in: u16 addr, u8 data 
#define CMD_KEYBOARD_CMD                 0x62  // in: u8, out: ? 
#define CMD_TOUCHPAD_CMD                 0x63  // in: u8, out: ? 
#define CMD_GET_FW_HASH                  0x64  // out: u8 * 16 
#define CMD_SUSPEND_HINT                 0x65  // in: u8
#define CMD_ENABLE_WAKE_TIMER            0x66  // in: u8
#define CMD_SET_WAKE_TIMER               0x67  // in: 32
#define CMD_ENABLE_WAKE_AUTORESET        0x68  // in: u8
#define CMD_OLS_SET_LIMITS               0x69  // in: u16, u16
#define CMD_OLS_GET_LIMITS               0x6a  // out: u16, u16
#define CMD_OLS_SET_CEILING              0x6b  // in: u16
#define CMD_OLS_GET_CEILING              0x6c  // out: u16
#define CMD_READ_HOST_SUS_PWR            0x6d  // out: s16
#define CMD_SET_USB_POWER                0x6e  // in: u8
// reserved for EXT SD power   0x6f
#define CMD_READ_BATTERY_STATUS16        0x70  // out: u16, bitmask
#define CMD_READ_BATTERY_CAPACITY        0x71  // out: u16, bitmask
# define EC_BAT_STATUS16 CMD_READ_BATTERY_STATUS16



/*
 * errors reported from the EC
 */
#define ERR_EC_BAD_COMMAND		1



/*
 * EC events
 */
#define EVENT_AC_CHANGE            1    // AC plugged/unplugged 
#define EVENT_BATTERY_STATUS       2    // Battery low/full/error/gone 
#define EVENT_BATTERY_CRITICAL     3    // Battery critical voltage 
#define EVENT_BATTERY_SOC_CHANGE   4    // 1% SOC Change 
#define EVENT_BATTERY_ERROR        5    // "Abnormal" error, query for cause 
#define EVENT_POWER_PRESSED        6    // power button was pressed 
#define EVENT_POWER_PRESS_WAKE     7    // power button pressed during sleep
#define EVENT_TIMED_HOST_WAKE      8    // Host wake timer
#define EVENT_OLS_HIGH_LIMIT       9    // OLS crossed dark threshold
#define EVENT_OLS_LOW_LIMIT       10    // OLS crossed light threshold

#define MMP2_GPU_PHYS_BASE		0xd420d000

#endif
