/*
 * Driver for the Neonode zForce IR touchscreen controller.
 * (as implemented on the OLPC XO laptop)
 *
 *   Copyright (C) 2010 One Laptop per Child Foundation.
 *
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU  General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This file is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 */


#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/input.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/kthread.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#include <linux/seq_file.h>
#include <linux/io.h>
#include <linux/gpio.h>
#include <linux/of_gpio.h>
#include <linux/delay.h>
#include <linux/debugfs.h>
#include <mach/mfp-mmp2.h>
#include <linux/input/mt.h>
#include <mach/olpc-xo-1-75.h>


#include "zforce.h"

#define DRIVER_NAME		"zforce"


// debug level
static int debug;
module_param(debug, int, 0644);

// if unset, send traditional events rather than multitouch
static bool multitouch = 1;
module_param(multitouch, bool, 0644);

// flip the axis orientation(s)
static int flip_x = 0;
module_param(flip_x, int, 0644);
static int flip_y = 0;
module_param(flip_y, int, 0644);

// scale the axes (in tenths of a percent, so 1000 means no scaling)
// this allows scaling "outward" by a bit.  this can make it possible for
// an average finger to point at pixels right at the edge of the screen,
// where the bezel makes it difficult otherwise.  overall accuracy suffers.
static int scale_x = 1000;
module_param(scale_x, int, 0644);
static int scale_y = 1000;
module_param(scale_y, int, 0644);

// set jitter to 0 for fully raw event reporting
static unsigned int jitter = 1;
module_param(jitter, uint, 0644);

// ignore the host-disable signal
static unsigned int forceload = 0;
module_param(forceload, uint, 0644);

// static priv pointer needed for the dynamic module parameters
static struct zforce_priv *zforce_param_priv;
int zforce_modify_scan_freq(struct zforce_priv *priv);
int zforce_modify_touch_size(struct zforce_priv *priv);

static char *ts_name[] = { "down", "move", "up", "invalid"};

static int param_set_scan_freq(const char *val, const struct kernel_param *kp)
{
	int ret = param_set_int(val, kp);
	if (ret)
		return ret;

	if (!zforce_param_priv) // during module load
		return 0;

	zforce_modify_scan_freq(zforce_param_priv);

	return 0;
}

static const struct kernel_param_ops param_ops_scan_freq = {
	.set = param_set_scan_freq,
	.get = param_get_int,
};

// override the low power "no touch detected yet" scanning frequency
static int idlescanfreq = 20; // hz
module_param_cb(idlescanfreq, &param_ops_scan_freq, &idlescanfreq, 0644);

// override the "touch is active" scanning frequency
static int fullscanfreq = 60; // hz
module_param_cb(fullscanfreq, &param_ops_scan_freq, &fullscanfreq, 0644);


static int param_set_touch_size(const char *val, const struct kernel_param *kp)
{
	int ret = param_set_int(val, kp);
	if (ret)
		return ret;

	if (!zforce_param_priv) // during module load
		return 0;

	zforce_modify_touch_size(zforce_param_priv);

	return 0;
}

static const struct kernel_param_ops param_ops_touch_size = {
	.set = param_set_touch_size,
	.get = param_get_int,
};

// if non-zero, the smallest allowable dimension of a permissible touch
static int mintouchsize = 0; // mm
module_param_cb(mintouchsize, &param_ops_touch_size, &mintouchsize, 0644);

// if non-zero, the largest allowable dimension of a permissible touch
static int maxtouchsize = 30; // mm
module_param_cb(maxtouchsize, &param_ops_touch_size, &maxtouchsize, 0644);


// these are arbitrary -- we tell the firmware our dimensions
#define MAX_WIDTH  1200
#define MAX_HEIGHT  900

#define MAX_TOUCHES 4

#define ZF_FRAMESTART 0xee

#define ZF_SUCCESS 0	    // successful command completion

#define ZF_ASIC_ERROR 0xe0  // from BootComplete: no NN1001 ASIC detected,
			    // which likely means the IR PCB isn't connected

#define I2C_RETRY_COUNT		3

struct olpc_zforce_platform_data {
	int	gpio_reset;		// active low
	int	gpio_test;		// unused, part of BSL handshake
	int	gpio_host_disable;	// when high, host tri-states all gpios
	int	gpio_data_ready;	// low when controller has data for us
};

/* the data format is the same for both commands and responses.
 * when receiving, the first two bytes are read, to get the size
 * for the second read, then the rest.
 */
struct zf_frame {
	unsigned char framestart;	// 0xee
	unsigned char datasize;		// actual length of payload
	struct {
		unsigned char id;	// command/response identifier
		unsigned char data[254];
	} payload;
};


struct zforce_priv {
	struct i2c_client *client;
	struct input_dev *input;

	struct {
		int active, x, y, size;
	} slot[MAX_TOUCHES];

	/* there's just one command and one response in play at a time. */
	struct zf_frame cmd_frame;
	struct zf_frame rcv_frame;

	/* where bulky command responses go */
	char *caller_buf;
	int caller_len;

	int max_x;
	int max_y;

	int gpio_reset;
	int gpio_test;
	int gpio_host_disable;
	int gpio_data_ready;
	int dr_irq;

	struct completion cmd_done;
	unsigned char cmd_reply_id;
	int cmd_status;
	int pulse_strength;

	struct zforce_debugfs *debugfs;

	int use_ints;
	struct workqueue_struct *irq_work_queue;
	struct work_struct irq_work;
#ifdef CONFIG_MACH_OLPC_XO_1_75
	int save_mfpr;
#endif

	struct SettingsResponse settings;
};

static void setup_debugfs(struct zforce_priv *);
static void cleanup_debugfs(struct zforce_priv *);

static const struct i2c_device_id zforce_i2c_id[] = {
	{ DRIVER_NAME, 0}, {}
};

MODULE_DEVICE_TABLE(i2c, zforce_i2c_id);

DEFINE_MUTEX(zforce_hw_mutex);
DEFINE_MUTEX(zforce_cmd_mutex);

void
slot_clear(struct zforce_priv *priv, int s)
{
	priv->slot[s].active = 0;
	priv->slot[s].x =
	priv->slot[s].y =
	priv->slot[s].size = -10000; // ensure initial deltas are huge
}

int
zforce_touch_motion(struct zforce_priv *priv, struct TouchInfo *ti,
		int slot, int state)
{
	int x, y, size;
	int delta_x, delta_y, delta_s;

	x = (ti->xhigh << 8) | ti->xlow;
	y = (ti->yhigh << 8) | ti->ylow;

	if (flip_x) x = MAX_WIDTH - x;
	if (flip_y) y = MAX_HEIGHT - y;

	x = ((x - MAX_WIDTH/2) * scale_x / 1000) + MAX_WIDTH/2;
	if (x < 0) x = 0;
	else if (x >= MAX_WIDTH) x = MAX_WIDTH - 1;

	y = ((y - MAX_HEIGHT/2) * scale_y / 1000) + MAX_HEIGHT/2;
	if (y < 0) y = 0;
	else if (y >= MAX_HEIGHT) y = MAX_HEIGHT - 1;

	size = (ti->width + ti->height)/2;

	if (!multitouch) {
		if (debug) {
			printk("zforce: legacy x %d y %d %s\n",
				x, y, ts_name[state]);
		}
		input_report_abs(priv->input, ABS_X, x);
		input_report_abs(priv->input, ABS_Y, y);
		input_report_key(priv->input, BTN_TOUCH, 1);
		input_sync(priv->input);
		return 1;
	}

	if (debug) {
		printk("zforce: mt x %d y %d size %d slot %d %s\n",
			x, y, size, slot, ts_name[state]);
	}

	input_mt_slot(priv->input, slot);

	/* suppress redundant events */
	if (!priv->slot[slot].active ) {
		int tid = input_mt_new_trkid(priv->input);
		input_report_abs(priv->input, ABS_MT_TRACKING_ID, tid);
		priv->slot[slot].active = 1;
		if (debug)
			printk("zforce: new tid %d slot %d\n", tid, slot);
	}

	delta_x = abs(x - priv->slot[slot].x);
	delta_y = abs(y - priv->slot[slot].y);
	delta_s = abs(size - priv->slot[slot].size);

	// if any delta exceeds the jitter threshold, report them all
	if (delta_x > jitter || delta_y > jitter || delta_s > jitter) {
		if (delta_x > 0) {
			input_report_abs(priv->input, ABS_MT_POSITION_X, x);
			priv->slot[slot].x = x;
		}
		if (delta_y > 0) {
			input_report_abs(priv->input, ABS_MT_POSITION_Y, y);
			priv->slot[slot].y = y;
		}
		if (delta_s > 0) {
			input_report_abs(priv->input, ABS_MT_PRESSURE, size * size);
			priv->slot[slot].size = size;
		}

		input_mt_sync(priv->input);

		return 1;
	}

	return 0;
}


int
zforce_touch_done(struct zforce_priv *priv, int slot, int state)
{
	if (!multitouch) {
		if (debug)
			printk("zforce: legacy %s\n", ts_name[state]);
		input_report_key(priv->input, BTN_TOUCH, 0);
		input_sync(priv->input);
		return 1;
	}

	if (debug)
		printk("zforce: slot %d %s\n", slot, ts_name[state]);

	input_mt_slot(priv->input, slot);
	input_report_abs(priv->input, ABS_MT_TRACKING_ID, -1);
	input_mt_sync(priv->input);
	slot_clear(priv, slot);
	return 1;
}

void
zforce_touchdata(struct zforce_priv *priv, struct TouchResponse *t)
{
	int i;
	int reported = 0;

	for (i = 0; i < t->count; i++) {
		int touch_id, touch_state;
		struct TouchInfo *ti = &t->touch[i];

		touch_state = (ti->id_state & ZF_TOUCHSTATE_MASK);
		touch_id = (ti->id_state >> 2);
		if (touch_id == 0 || touch_id > MAX_TOUCHES) {
			printk(KERN_WARNING "%s: touch_id out of range: %d\n",
				__func__, t->count);
			continue;
		}

		switch(touch_state) {
		case ZF_TOUCHSTATE_DOWN:
		case ZF_TOUCHSTATE_MOVE:
			reported = zforce_touch_motion(priv, ti, touch_id - 1, touch_state);
			break;

		case ZF_TOUCHSTATE_INVALID:
		case ZF_TOUCHSTATE_UP:
			reported = zforce_touch_done(priv, touch_id - 1, touch_state);
			break;
		}
	}

	if (reported)
		input_sync(priv->input);
}

int
zforce_receive(struct zforce_priv *priv)
{
	int ret;

	mutex_lock(&zforce_hw_mutex);

	ret = i2c_master_recv(priv->client, (char *)&priv->rcv_frame, 2);
	if (ret < 0) {
		printk(KERN_ERR "%s: i2c recv of framestart failed, %d\n", __func__, ret);
		goto out;
	}

	if (priv->rcv_frame.framestart != ZF_FRAMESTART ||
			!priv->rcv_frame.datasize) {
		printk(KERN_ERR "%s: received bad framestart 0x%x, datasize %d\n",
			__func__, priv->rcv_frame.framestart, priv->rcv_frame.datasize);
		ret = -1;
		goto out;
	}

	ret = i2c_master_recv(priv->client, (char *)&priv->rcv_frame.payload,
			priv->rcv_frame.datasize);
	if (ret < 0) {
		printk(KERN_ERR "%s: i2c recv of data failed, %d\n", __func__, ret);
		goto out;
	}

    out:
	mutex_unlock(&zforce_hw_mutex);
	return ret;
}

static int
zforce_process(struct zforce_priv *priv, int completion)
{
	int ret;

	ret = zforce_receive(priv);
	if (ret < 0) {
		return ret;
	}

	/* all command/response pairs are one-for-one except for
	 * TouchData (which enables the touch responses, which
	 * will continue until Deactivate is sent) and
	 * BootComplete, which will always be the first response
	 * received after a reset -- no other commands will be
	 * handled until it is read.
	 */

	priv->cmd_reply_id = priv->rcv_frame.payload.id;

	if (debug > 1)
		printk("zforce: reply id 0x%x\n", priv->cmd_reply_id);

	switch(priv->cmd_reply_id) {

	// asynchronous -- not in response to a command
	case ZF_TouchData:
		zforce_touchdata(priv,
			(struct TouchResponse *)&priv->rcv_frame.payload.data);
		completion = 0;
		break;


	// simple commands -- return just a pass/fail response.
	case ZF_InvalidCommand:
		printk(KERN_WARNING "%s: invalid command response 0x%02x\n",
			__func__, priv->rcv_frame.payload.data[0]);
		// fallthrough
	case ZF_BootComplete:
	case ZF_Deactivate:
	case ZF_Activate:
	case ZF_Resolution:
	case ZF_Configuration:
	case ZF_ScanningFrequency:
	case ZF_TouchAreaSizes:
	case ZF_ForceCalibration:
	case ZF_ActiveLeds:
		priv->cmd_status = priv->rcv_frame.payload.data[0];
		break;

	// "bulky" commands -- have data to return
	case ZF_StatusCommand:
	case ZF_FixedPulseStrength:
	case ZF_LEDLevels:
	case ZF_ForcedLEDLevels:
	case ZF_LowSignals:
	case ZF_OpenShort:
		if (priv->caller_len && priv->caller_buf) {
			memcpy(priv->caller_buf, &priv->rcv_frame.payload.data,
				min(priv->caller_len, priv->rcv_frame.datasize-1));
			priv->cmd_status = ZF_SUCCESS;
		} else {
			printk(KERN_WARNING "%s: no place for data for 0x%x\n",
				__func__, priv->rcv_frame.payload.id);
			priv->cmd_status = !ZF_SUCCESS;
		}
		break;

	// errors
	case ZF_Overrun:
		printk(KERN_WARNING "%s: overrun reported, command lost\n", __func__);
		priv->cmd_status = -1;
		break;

	default:
		printk(KERN_WARNING "%s: unhandled command response 0x%02x\n",
			__func__, priv->rcv_frame.payload.data[0]);
		completion = 0;
		break;
	}

	if (completion)
		complete(&priv->cmd_done);

	return ret;
}

static inline int
zforce_data_ready(struct zforce_priv *priv)
{
	// Data Ready is active low (and is also the interrupt line)
	return !gpio_get_value(priv->gpio_data_ready);
}

static void
zforce_i2c_irq_work_queue_func(struct work_struct *work)
{
	struct zforce_priv *priv =
		container_of(work, struct zforce_priv, irq_work);

	// irq transitions low, and stays low while a touch is detected
	while (zforce_data_ready(priv)) {
		if (zforce_process(priv, 1) < 0) {
			msleep(3000);
			printk(KERN_ERR "%s: process error\n", __func__);
		}

		if (kthread_should_stop()) {
			break;
		}
	}

	enable_irq(priv->dr_irq);
}


static irqreturn_t
zforce_handler(int irq, void *dev_id)
{
	struct zforce_priv *priv = dev_id;

	disable_irq_nosync(irq);
	queue_work(priv->irq_work_queue, &priv->irq_work);

	return IRQ_HANDLED;
}


void
zforce_gpio_free(struct zforce_priv *priv)
{
	gpio_direction_input(priv->gpio_reset);
	gpio_direction_input(priv->gpio_test);
	gpio_direction_input(priv->gpio_host_disable);
	gpio_direction_input(priv->gpio_data_ready);
	gpio_free(priv->gpio_reset);
	gpio_free(priv->gpio_test);
	gpio_free(priv->gpio_host_disable);
	gpio_free(priv->gpio_data_ready);
}

static struct olpc_zforce_platform_data olpc_zforce_platform_data;
struct olpc_zforce_platform_data *
olpc_zforce_get_of_platform_data(struct device *dev)
{
	struct olpc_zforce_platform_data *pd = &olpc_zforce_platform_data;
	struct device_node *np = dev->of_node;

	pd->gpio_reset		= of_get_named_gpio(np, "reset-gpios", 0);
	pd->gpio_test		= of_get_named_gpio(np, "test-gpios", 0);
	pd->gpio_host_disable	= of_get_named_gpio(np, "hd-gpios", 0);
	pd->gpio_data_ready	= of_get_named_gpio(np, "dr-gpios", 0);

	return pd;
}



int
zforce_gpio_requests(struct zforce_priv *priv)
{
	struct olpc_zforce_platform_data *pd;
	int ret;

#ifdef CONFIG_OF
	pd = olpc_zforce_get_of_platform_data(&priv->client->dev);
	if (!pd) {
		return -1;
	}
#else
	pd = priv->client->dev.platform_data;
#endif

	ret = gpio_request_one(pd->gpio_reset,
			GPIOF_OUT_INIT_HIGH, "zforce reset");
	if (ret < 0) {
		return ret;
	}
	ret = gpio_request_one(pd->gpio_test,
			GPIOF_OUT_INIT_LOW, "zforce test");
	if (ret < 0) {
		gpio_free(pd->gpio_reset);
		return ret;
	}
	ret = gpio_request_one(pd->gpio_host_disable,
			GPIOF_IN, "zforce host disable");
	if (ret < 0) {
		gpio_free(pd->gpio_reset);
		gpio_free(pd->gpio_test);
		return ret;
	}
	ret = gpio_request_one(pd->gpio_data_ready,
			GPIOF_IN, "zforce data ready");
	if (ret < 0) {
		gpio_free(pd->gpio_reset);
		gpio_free(pd->gpio_test);
		gpio_free(pd->gpio_host_disable);
		return ret;
	}

	priv->gpio_reset = pd->gpio_reset;
	priv->gpio_test = pd->gpio_test;
	priv->gpio_host_disable = pd->gpio_host_disable;
	priv->gpio_data_ready = pd->gpio_data_ready;
	priv->dr_irq = gpio_to_irq(priv->gpio_data_ready);

	return 0;
}

int
zforce_wait_receive(struct zforce_priv *priv, unsigned char expected)
{
	int i = 0;
	int ret;

	if (priv->use_ints) {
		if (!wait_for_completion_timeout(&priv->cmd_done,
				msecs_to_jiffies(2000))) {
			printk(KERN_ERR "%s: timeout expecting 0x%02x\n", __func__, expected);
			return -ETIMEDOUT;
		}
	} else {
		// polling is useful before we first enable interrupts
		while (!zforce_data_ready(priv)) {
			msleep(10);
			if (i++ > 200) {
				printk(KERN_ERR "%s: timed out expecting 0x%02x\n", __func__, expected);
				return -ETIMEDOUT;
			}
		}

		ret = zforce_process(priv, 0);
		if (ret < 0) {
			return ret;
		}

	}

	if (priv->cmd_reply_id != expected) {
		printk(KERN_ERR "%s: expected 0x%02x, got 0x%02x\n",
			__func__, expected, priv->cmd_reply_id);
		return -1;
	}

	return 0;
}

int
zforce_write(struct i2c_client *client, unsigned char *data, int len)
{
	int ret;

	mutex_lock(&zforce_hw_mutex);

	if ((ret = i2c_master_send(client, data, len)) != len) {
		dev_err(&client->dev, "%s: i2c send failed\n", __func__);
	} else {
		ret = 0;
	}

	mutex_unlock(&zforce_hw_mutex);
	return ret;
}

int
zforce_command(struct zforce_priv *priv, unsigned char cmd,
	char *data, int len, char *rdata, int rlen)
{
	int ret;

	mutex_lock(&zforce_cmd_mutex);

	memcpy(priv->cmd_frame.payload.data, data, len);
	priv->cmd_frame.payload.id = cmd;
	priv->cmd_frame.datasize = len + 1;
	priv->cmd_frame.framestart = ZF_FRAMESTART;

	priv->rcv_frame.payload.id = 0xff;

	priv->caller_buf = rdata;
	priv->caller_len = (rlen >= 0) ? rlen : 0;


	if (debug > 1)
		printk("zforce: command 0x%x, write %d, read %d\n", cmd, len, rlen);

	ret = zforce_write(priv->client, (char *)&priv->cmd_frame, (int)sizeof(struct zf_frame));
	if (ret < 0) {
		mutex_unlock(&zforce_cmd_mutex);
		return ret;
	}

	// negative rlen means we won't wait for a response at all
	if (rlen < 0) {
		mutex_unlock(&zforce_cmd_mutex);
		return 0;
	}

	// exception: the response to ZF_ForcedLEDLevels is ZF_LEDLevels
	ret = zforce_wait_receive(priv,
		(cmd == ZF_ForcedLEDLevels) ? ZF_LEDLevels : cmd);
	if (ret < 0) {
		mutex_unlock(&zforce_cmd_mutex);
		return ret;
	}

	if (priv->cmd_status != ZF_SUCCESS) {
		printk(KERN_INFO "%s: command 0x%02x, returned 0x%02x\n",
			__func__, cmd, priv->cmd_status);
		mutex_unlock(&zforce_cmd_mutex);
		return -1;
	}

	mutex_unlock(&zforce_cmd_mutex);
	return 0;

}

int
zforce_reset(struct zforce_priv *priv)
{
	int ret;

	gpio_set_value(priv->gpio_reset, 0);
	udelay(50);
	gpio_set_value(priv->gpio_reset, 1);
	msleep(100);

	priv->rcv_frame.payload.id = 0xff;

	ret = zforce_wait_receive(priv, ZF_BootComplete);
	if (ret < 0) {
		return ret;
	}

	ret = priv->cmd_status;
	if (ret == ZF_ASIC_ERROR) {
		printk(KERN_ERR "%s: ASIC error: no touchscreen connected?\n", __func__);
	} else if (ret != ZF_SUCCESS) {
		printk(KERN_ERR "%s: unsuccessful BootComplete: 0x%02x\n", __func__, ret);
	}

	return ret;
}

int
zforce_input_setup(struct zforce_priv *priv)
{
	struct input_dev *input;
	int ret;

	input = input_allocate_device();
	if (input == NULL) {
		printk(KERN_ERR "%s: allocate input device, error\n", __func__);
		return -ENOMEM;
	}

	input->evbit[0] = BIT_MASK(EV_ABS);


	/* these seem like they should only be needed for legacy
	 * mode, but we won't be recognized as a touchscreen (by
	 * either udev or evdev) uless we have all three.
	 */
	input->evbit[0] |= BIT_MASK(EV_KEY);
	input->keybit[BIT_WORD(BTN_TOUCH)] = BIT_MASK(BTN_TOUCH);
	input_set_abs_params(input, ABS_X, 0, priv->max_x, 0, 0);
	input_set_abs_params(input, ABS_Y, 0, priv->max_y, 0, 0);

	if (multitouch) {
		input_set_abs_params(input, ABS_MT_POSITION_X, 0, priv->max_x, 0, 0);
		input_set_abs_params(input, ABS_MT_POSITION_Y, 0, priv->max_y, 0, 0);
		input_set_abs_params(input, ABS_MT_PRESSURE, 0, 255, 0, 0);
		input_mt_init_slots(input, MAX_TOUCHES);
	}

	input->name = "zForce touchscreen";
	input->id.bustype = BUS_I2C;
	input->dev.parent = &priv->client->dev;

	printk(KERN_INFO "zforce: running in %stouch mode\n",
		multitouch ? "multi":"single ");

	ret = input_register_device(input);
	if (ret) {
		printk(KERN_ERR "%s: register input device, error\n", __func__);
		input_free_device(input);
		return ret;
	}

	priv->input = input;
	input_set_drvdata(input, priv);

	return 0;
}

static int
zforce_i2c_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	struct zforce_priv *priv;
	int ret, zf_status, s;
	struct SizeInfo si;
	struct ConfigurationInfo ci;

	priv = kzalloc(sizeof(*priv), GFP_KERNEL);
	if (!priv) {
		dev_err(&client->dev, "%s: allocation failure\n", __func__);
		return -ENOMEM;
	}

	priv->client = client;
	priv->max_x = MAX_WIDTH;
	priv->max_y = MAX_HEIGHT;

	for (s = 0; s < MAX_TOUCHES; s++)
		slot_clear(priv, s);

	if ((client->addr == 0) || (client->adapter == 0)
		|| (client->driver == 0)) {
		printk(KERN_ERR "%s: i2c configuration invalid\n", __func__);
		ret = -1;
		goto out4;
	}

	ret = zforce_gpio_requests(priv);
	if (ret) {
		printk(KERN_ERR "%s: failed to request gpios\n", __func__);
		goto out4;
	}

	// if TOUCH_HD# ("host disable") is asserted low, we assume that
	// either the touchscreen controller is missing, or debug mode is
	// being requested via neonode's connector.  early (B1) XO-4 laptops
	// are excluded because many have clear lightguides which demand
	// older firmware which drives the TOUCH_HD# signal incorrectly.
	if (gpio_get_value(priv->gpio_host_disable) == 0 && !forceload &&
		    olpc_board_at_least(olpc_board(0x4c1))) 
	{
		printk("zforce: host-disable asserted, skipping driver load\n");
		printk("zforce: set forceload=1 to override\n");
		ret = -ENODEV;
		goto out3;
	}

	zf_status = zforce_reset(priv);
	if (zf_status < 0) {
		ret = zf_status;
		goto out3;
	}

	ret = zforce_command(priv, ZF_StatusCommand, NULL, 0,
		(char *)&priv->settings, sizeof(priv->settings));
	if (ret) {
		goto out3;
	}

	printk("zforce: found zForce firmware %d.%d.%d.%d\n",
		priv->settings.major, priv->settings.minor,
		priv->settings.build, priv->settings.revision);

	if (zf_status != ZF_SUCCESS) {  // probably ZF_ASIC_ERROR 
		ret = -ENXIO;
		goto out3;
	}

	ret = zforce_input_setup(priv);
	if (ret) {
		goto out3;
	}

	init_completion(&priv->cmd_done);

	priv->irq_work_queue =
		create_singlethread_workqueue("zforce_i2c_irq_queue");
	if (!priv->irq_work_queue) {
		printk(KERN_ERR "%s: failed to create workqueue\n", __func__);
		ret = -ENOMEM;
		goto out2;
	}

	INIT_WORK(&priv->irq_work, zforce_i2c_irq_work_queue_func);

	i2c_set_clientdata(client, priv);

	ret = request_irq(priv->dr_irq, zforce_handler,
					IRQF_DISABLED | IRQF_TRIGGER_FALLING,
					"zforce_i2c_irq", priv);
	if (ret) {
		printk(KERN_ERR "%s: request irq, error\n", __func__);
		goto out1;
	}

	priv->use_ints = 1;

	setup_debugfs(priv);

	// we can drop this check eventually -- early firmware has X flipped
	if (priv->settings.major == 0 &&
	    priv->settings.minor == 0 &&
	    priv->settings.build == 0 &&
	    priv->settings.revision <= 5) {
		flip_x = 1;
	}

	// send Activate
	ret = zforce_command(priv, ZF_Activate, NULL, 0, NULL, 0);
	if (ret) {
		goto out;
	}

	// send Resolution
	si.width = priv->max_x;
	si.height = priv->max_y;
	ret = zforce_command(priv, ZF_Resolution, (char *)&si, sizeof(si), NULL, 0);
	if (ret) {
		goto out;
	}

	// send Configuration
	ci.flags = ZF_FLAG_DUALTOUCH;
	ret = zforce_command(priv, ZF_Configuration, (char *)&ci, sizeof(ci), NULL, 0);
	if (ret) {
		goto out;
	}


	zforce_param_priv = priv;  // for later dynamic parameter changes

	// set scanning frequencies and touch area parameters
	ret = zforce_modify_scan_freq(priv);
	if (ret) {
		goto out;
	}

	ret = zforce_modify_touch_size(priv);
	if (ret) {
		goto out;
	}


	// finally send TouchDataRequest to start things up
	ret = zforce_command(priv, ZF_TouchData, NULL, 0, NULL, -1);
	if (ret) {
		goto out;
	}

	if (!device_can_wakeup(&client->dev))
		device_init_wakeup(&client->dev, 1);

	return 0;

    out:

	disable_irq(priv->dr_irq);
	free_irq(priv->dr_irq, priv);
	cleanup_debugfs(priv);

    out1:
	destroy_workqueue(priv->irq_work_queue);
	priv->irq_work_queue = 0;

    out2:
	input_unregister_device(priv->input);
	priv->input = 0;

    out3:
	zforce_gpio_free(priv);

    out4:
	i2c_set_clientdata(client, 0);
	kfree(priv);
	printk(KERN_ERR "%s: failed\n", __func__);
	return ret;
}

static int __devexit
zforce_i2c_remove(struct i2c_client *client)
{
	struct zforce_priv *priv = i2c_get_clientdata(client);

	cleanup_debugfs(priv);

	disable_irq(priv->dr_irq);
	destroy_workqueue(priv->irq_work_queue);
	free_irq(priv->dr_irq, priv);

	input_unregister_device(priv->input);
	zforce_gpio_free(priv);

	return 0;
}

int zforce_modify_scan_freq(struct zforce_priv *priv)
{
	struct ScanningFrequencyInfo sfi;

	sfi.idleFrequency = idlescanfreq;
	sfi.fullScanningFrequency = fullscanfreq;
	sfi.penFrequency = 0;

	return zforce_command(priv, ZF_ScanningFrequency,
			(char *)&sfi, sizeof(sfi), NULL, 0);
}

int zforce_modify_touch_size(struct zforce_priv *priv)
{
	struct TouchAreaSizes tas;

	if (mintouchsize) {
		tas.min.enabled = 1;
		tas.min.size = mintouchsize;
	} else {
		tas.min.enabled = tas.min.size = 0;
	}
	if (maxtouchsize) {
		tas.max.enabled = 1;
		tas.max.size = maxtouchsize;
	} else {
		tas.max.enabled = tas.max.size = 0;
	}
	return zforce_command(priv, ZF_TouchAreaSizes,
			(char *)&tas, sizeof(tas), NULL, 0);
}

#ifdef CONFIG_TOUCHSCREEN_ZFORCE_DEBUG_FS

/*
 * in OLPC's hardware implementation, there are 11 IR LEDs facing
 * 12 photodetectors in Y, and 15 IR LEDs facing 16 photodetectors in Y.
 *
 *                       X
 *           ir ir ir            ir ir
 *           12 26 25 .......... 14 13
 *          +-------------------------+
 *     ir11 |                         |  pd12
 *      .   |                         |  .
 *      .   |                         |  .
 *   Y  .   |                         |  .
 *      .   |                         |  .
 *      .   |                         |  .
 *     ir2  |                         |  pd2
 *     ir1  |                         |  pd1
 *          +-------------------------+
 *          pd pd ........... pd pd pd
 *          28 27             15 14 13
 *
 * yes, the numbering really does do that funky thing with ir12
 * being around the corner from where it looks like it should be.
 * don't ask me why.  luckily, that probably doesn't matter
 * unless you're looking at output dumped from one of the debugfs
 * nodes (and even then i'm not sure what affect it has).  in
 * normal operation that's all hidden.
 *
 * regarding the numAxisSignals value returned from the API:  as
 * explained to me, "A signal is a LED/PD-pair.  In almost all
 * cases, a LED fires on two photodetectors (PDs) meaning two
 * signals per LED.  For instance, in LEDLevels, the LED is
 * indexed and the data you receive is for both signals from that
 * LED (LEDStrength and PDSignal)."
 */

static int
zforce_debugfs_null_open(struct inode *inode, struct file *f)
{
	f->private_data = inode->i_private;
	return 0;
}

static ssize_t
zforce_debugfs_calibrate_write(struct file *file, const char __user *user_buf,
				size_t count, loff_t *ppos)
{
	struct zforce_priv *priv;
	char buf[4];
	int ret;

	priv = file->private_data;

	// "echo > calibration"  to trigger the calibration
	count = simple_write_to_buffer(buf, sizeof(buf), ppos, user_buf, count);
	if (count != 1 || buf[0] != '\n')
		return -EINVAL;

	ret = zforce_command(priv, ZF_ForceCalibration, NULL, 0, NULL, 0);
	printk("%s: ZF_ForceCalibration returned %d\n", __func__, ret);
	if (ret < 0)
		return ret;

	return count;
}

static int
zforce_show_fixedpulse_axis(struct seq_file *seq, char axis)
{
	struct zforce_priv *priv = seq->private;
	struct FixedPulseInfo fpi;
	struct FixedPulseStrengthResponse fpsr;
	int ret, i, pd;

	axis -= 'x';
	fpi.axis = axis;
	fpi.strength = priv->pulse_strength;

	ret = zforce_command(priv, ZF_FixedPulseStrength,
			(char *)&fpi, sizeof(fpi), (char *)&fpsr, sizeof(fpsr));
	if (ret < 0) {
		printk("%s: ZF_FixedPulseStrength failed\n", __func__);
		return ret;
	}

	seq_printf(seq, "%c PD signals:\n  ", axis + 'x');
	for (i = 0; i < fpsr.numAxisSignals; i++) {
		pd = fpsr.PDSignal[i];
		seq_printf(seq, "%02x ", pd);
		if (i % 8 == 7) seq_printf(seq, " ");
		if (i % 16 == 15) seq_printf(seq, "\n  ");
	}

	seq_printf(seq, "\n");

	return 0;
}

static int
zforce_show_fixedpulse(struct seq_file *seq, void *v)
{
	struct zforce_priv *priv = seq->private;

	if (priv->pulse_strength > 64) priv->pulse_strength = 64;
	else if (priv->pulse_strength < 1) priv->pulse_strength = 1;
	seq_printf(seq, "pulse strength: %d\n", priv->pulse_strength);

	zforce_show_fixedpulse_axis(seq, 'x');
	zforce_show_fixedpulse_axis(seq, 'y');

	return 0;
}

static ssize_t
zforce_debugfs_fixedpulse_write(struct file *f,
		const char __user *user_buf, size_t count, loff_t *ppos)
{
	struct zforce_priv *priv = ((struct seq_file *)f->private_data)->private;
	char buf[32];
	int n;

	// "echo 54 > fixedpulse"  to set level for subsequent scan
	count = simple_write_to_buffer(buf, sizeof(buf), ppos, user_buf, count);

	n = (int)simple_strtol(buf, NULL, 0);
	priv->pulse_strength = min(n, 64);

	return count;
}

static int
zforce_debugfs_fixedpulse_open(struct inode *inode, struct file *f)
{
	return single_open(f, zforce_show_fixedpulse, inode->i_private);
}

static int
zforce_show_settings(struct seq_file *seq, void *v)
{
	struct zforce_priv *priv = seq->private;
	struct SettingsResponse *s;
	int ret;


	ret = zforce_command(priv, ZF_StatusCommand, NULL, 0,
		(char *)&priv->settings, sizeof(priv->settings));
	if (ret < 0) {
		printk("%s: StatusCommand failed", __func__);
		return ret;
	}

	s = &priv->settings;

	seq_printf(seq, "firmware %d.%d.%d.%d\n",
		s->major, s->minor, s->build, s->revision);
	seq_printf(seq, "active %d, touches %d\n",
		!!(s->active_touchcount & 0x80), (s->active_touchcount & 0x7f));
	seq_printf(seq, "scanning %d, prepared %d, sent %d, invalid %d\n",
		s->scanningCounter, s->preparedTouchPackages,
		s->sentTouchPackages, s->invalidTouchCounter );
	seq_printf(seq, "config 0x%x\n", s->configuration.flags);
	seq_printf(seq, "freq: idle %d, full %d, pen %d\n",
		s->frequencyData.idleFrequency,
		s->frequencyData.fullScanningFrequency,
		s->frequencyData.penFrequency);
	seq_printf(seq, "resolution: w %d h %d, physical: w %d h %d\n",
		s->resolution.width, s->resolution.height,
		s->physicalSize.width, s->physicalSize.height);
	seq_printf(seq, "leds x %d-%d y %d-%d\n",
		s->activeLEDs.x_Start, s->activeLEDs.x_End,
		s->activeLEDs.y_Start, s->activeLEDs.y_End);
	seq_printf(seq, "maxstrength %d\n", s->maxStrength);
	seq_printf(seq, "touch area: en %d max %d, en %d min %d\n",
		s->maxSize.enabled, s->maxSize.size,
		s->minSize.enabled, s->minSize.size);

	return 0;
}

static int
zforce_debugfs_settings_open(struct inode *inode, struct file *file)
{
	return single_open(file, zforce_show_settings, inode->i_private);
}

static int
zforce_show_lowsignal_axis(struct seq_file *seq, char axis)
{
	struct zforce_priv *priv = seq->private;
	struct LowSignalResponse low;
	int nbits, byte, bindex, i;
	int ret;

	axis -= 'x';  // 0 or 1
	ret = zforce_command(priv, ZF_LowSignals,
			&axis, 1, (char *)&low, sizeof(low));
	if (ret < 0) {
		printk("%s: ZF_LowSignals failed\n", __func__);
		return ret;
	}

	nbits = low.numAxisSignals;

	seq_printf(seq, "%c: %d low signals:\n", axis + 'x', nbits);

	for (i = 0; i < nbits; i++) {
		byte = nbits / 8;
		bindex = nbits & 7;
		seq_printf(seq, "%d ", low.LowSignal[byte] & !!(1 << bindex));
		if (i % 8 == 7) seq_printf(seq, " ");
	}

	seq_printf(seq, "\n");

	return 0;
}

static int
zforce_show_lowsignal(struct seq_file *seq, void *v)
{

	zforce_show_lowsignal_axis(seq, 'x');
	zforce_show_lowsignal_axis(seq, 'y');

	return 0;
}


static int
zforce_debugfs_lowsignal_open(struct inode *inode, struct file *file)
{
	return single_open(file, zforce_show_lowsignal, inode->i_private);
}

static int
zforce_show_ledlevels_axis(struct seq_file *seq, char forced, char axis)
{
	struct zforce_priv *priv = seq->private;
	struct LEDLevelResponse ledlevels;
	int nlevels, i;
	int led, pd;
	int ret;

	axis -= 'x';  // 0 or 1
	ret = zforce_command(priv, forced ? ZF_ForcedLEDLevels : ZF_LEDLevels,
			&axis, 1, (char *)&ledlevels, sizeof(ledlevels));
	if (ret < 0) {
		printk("%s: ZF_[Forced]LedLevels failed\n", __func__);
		return ret;
	}

	nlevels = ledlevels.numAxisSignals;

	seq_printf(seq, "%c: %d levels:\n", axis + 'x', nlevels);

	seq_printf(seq, "led strength 1:\n  ");
	for (i = 0; i < nlevels; i++) {
		led = ledlevels.levels[i].LEDStrength & 0xf;
		seq_printf(seq, "%x ", led);
		if (i % 8 == 7) seq_printf(seq, " ");
		if (i % 16 == 15) seq_printf(seq, "\n  ");
	}

	seq_printf(seq, "\nled strength 2:\n  ");
	for (i = 0; i < nlevels; i++) {
		led = ledlevels.levels[i].LEDStrength >> 4;
		seq_printf(seq, "%x ", led);
		if (i % 8 == 7) seq_printf(seq, " ");
		if (i % 16 == 15) seq_printf(seq, "\n  ");
	}

	seq_printf(seq, "\nPD signal 1:\n  ");
	for (i = 0; i < nlevels; i++) {
		pd = ledlevels.levels[i].PDSignal1;
		seq_printf(seq, "%02x ", pd);
		if (i % 8 == 7) seq_printf(seq, " ");
		if (i % 16 == 15) seq_printf(seq, "\n  ");
	}
	seq_printf(seq, "\nPD signal 2:\n  ");
	for (i = 0; i < nlevels; i++) {
		pd = ledlevels.levels[i].PDSignal2;
		seq_printf(seq, "%02x ", pd);
		if (i % 8 == 7) seq_printf(seq, " ");
		if (i % 16 == 15) seq_printf(seq, "\n  ");
	}
	seq_printf(seq, "\n");

	return 0;
}

static int
zforce_show_ledlevels(struct seq_file *seq, void *v)
{

	zforce_show_ledlevels_axis(seq, 0, 'x');
	zforce_show_ledlevels_axis(seq, 0, 'y');

	return 0;
}


static int
zforce_debugfs_ledlevels_open(struct inode *inode, struct file *file)
{
	return single_open(file, zforce_show_ledlevels, inode->i_private);
}

static int
zforce_show_forcedledlevels(struct seq_file *seq, void *v)
{

	zforce_show_ledlevels_axis(seq, 1, 'x');
	zforce_show_ledlevels_axis(seq, 1, 'y');

	return 0;
}


static int
zforce_debugfs_forcedledlevels_open(struct inode *inode, struct file *file)
{
	return single_open(file, zforce_show_forcedledlevels, inode->i_private);
}

static int
zforce_show_openshort_axis(struct seq_file *seq, char axis)
{
	struct zforce_priv *priv = seq->private;
	struct OpenShortResponse os;
	int nLEDbits, nPDbits, totbits, byte, bindex, i;
	int ret;

	axis -= 'x';  // 0 or 1
	ret = zforce_command(priv, ZF_OpenShort,
			&axis, 1, (char *)&os, sizeof(os));
	if (ret < 0) {
		printk("%s: ZF_OpenShort failed\n", __func__);
		return ret;
	}

	/*
	 * if you have one more PD than LED on the axis, then the
	 * number of LEDs is NumberOfSignals/2, and the number of
	 * PDs is NumberOfLEDs + 1.
	 */
	nLEDbits = os.numAxisSignals / 2;
	nPDbits = nLEDbits + 1;

	totbits = 2*nPDbits + 2*nLEDbits;
	seq_printf(seq, "\n%c: open photo detectors:\n  ", axis + 'x');
	for (i = 0; i < nPDbits; i++, totbits++) {
		byte = totbits / 8;
		bindex = totbits & 7;
		seq_printf(seq, "%d ", os.bits[byte] & !!(1 << bindex));
		if (i % 4 == 3) seq_printf(seq, " ");
	}

	seq_printf(seq, "\n%c: shorted photo detectors:\n  ", axis + 'x');
	for (i = 0; i < nPDbits; i++, totbits++) {
		byte = totbits / 8;
		bindex = totbits & 7;
		seq_printf(seq, "%d ", os.bits[byte] & !!(1 << bindex));
		if (i % 4 == 3) seq_printf(seq, " ");
	}
	seq_printf(seq, "\n%c: shorted leds:\n  ", axis + 'x');
	for (i = 0; i < nLEDbits; i++, totbits++) {
		byte = totbits / 8;
		bindex = totbits & 7;
		seq_printf(seq, "%d ", os.bits[byte] & !!(1 << bindex));
		if (i % 4 == 3) seq_printf(seq, " ");
	}
	seq_printf(seq, "\n%c: open leds:\n  ", axis + 'x');
	for (i = 0; i < nLEDbits; i++, totbits++) {
		byte = totbits / 8;
		bindex = totbits & 7;
		seq_printf(seq, "%d ", os.bits[byte] & !!(1 << bindex));
		if (i % 4 == 3) seq_printf(seq, " ");
	}
	seq_printf(seq, "\n");
	return 0;
}

static int
zforce_show_openshort(struct seq_file *seq, void *v)
{

	zforce_show_openshort_axis(seq, 'x');
	zforce_show_openshort_axis(seq, 'y');

	return 0;
}

static int
zforce_debugfs_openshort_open(struct inode *inode, struct file *file)
{
	return single_open(file, zforce_show_openshort, inode->i_private);
}

static struct file_operations zforce_debugfs_calibrate_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_null_open,
	.write		= zforce_debugfs_calibrate_write,
};

static struct file_operations zforce_debugfs_fixedpulse_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_fixedpulse_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.write		= zforce_debugfs_fixedpulse_write,
	.release	= single_release,
};

static struct file_operations zforce_debugfs_forcedledlevels_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_forcedledlevels_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

static struct file_operations zforce_debugfs_ledlevels_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_ledlevels_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

static struct file_operations zforce_debugfs_lowsignal_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_lowsignal_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

static struct file_operations zforce_debugfs_openshort_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_openshort_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

static struct file_operations zforce_debugfs_settings_fops = {
	.owner		= THIS_MODULE,
	.open		= zforce_debugfs_settings_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= single_release,
};

struct zforce_debugfs {

	struct dentry *zforce_debugfs_dir;

};

static void
setup_debugfs(struct zforce_priv *priv)
{

	priv->debugfs = kzalloc(sizeof(struct zforce_debugfs), GFP_KERNEL);
	if (priv->debugfs == NULL)
		return;

	priv->debugfs->zforce_debugfs_dir = debugfs_create_dir("zforce", 0);
	if (priv->debugfs->zforce_debugfs_dir == ERR_PTR(-ENODEV)) {
		kfree(priv->debugfs);
		priv->debugfs = NULL;
		return;
	}

	debugfs_create_file("calibrate", 0200,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_calibrate_fops);
	debugfs_create_file("fixedpulse", 0600,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_fixedpulse_fops);
	debugfs_create_file("forcedledlevels", 0400,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_forcedledlevels_fops);
	debugfs_create_file("ledlevels", 0400,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_ledlevels_fops);
	debugfs_create_file("lowsignals", 0400,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_lowsignal_fops);
	debugfs_create_file("openshort", 0400,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_openshort_fops);
	debugfs_create_file("settings", 0444,
			priv->debugfs->zforce_debugfs_dir, priv,
			&zforce_debugfs_settings_fops);
}

static void
cleanup_debugfs(struct zforce_priv *priv)
{
	if (priv->debugfs->zforce_debugfs_dir)
		debugfs_remove_recursive(priv->debugfs->zforce_debugfs_dir);
}

#else

static void setup_debugfs(struct zforce_priv *priv) { }
static void cleanup_debugfs(struct zforce_priv *priv) { }

#endif // CONFIG_TOUCHSCREEN_ZFORCE_DEBUG_FS


#ifdef CONFIG_PM
static int
zforce_suspend(struct device *dev)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct zforce_priv *priv = i2c_get_clientdata(client);

	if (device_may_wakeup(&client->dev)) {
#ifdef CONFIG_MACH_OLPC_XO_1_75
		priv->save_mfpr = mfp_setedge(priv->gpio_data_ready, MFP_SETEDGE_BOTH);
#endif
		enable_irq_wake(priv->dr_irq);
	}
	else {
		int ret = zforce_command(priv, ZF_Deactivate, NULL, 0, NULL, 0);
		if (ret) {
			printk(KERN_WARNING "%s: ZF_Deactivate failed, ret %d\n",
				__func__, ret);
		}
	}

	disable_irq(priv->dr_irq);

	return 0;
}

static int
zforce_resume(struct device *dev)
{
	struct i2c_client *client = to_i2c_client(dev);
	struct zforce_priv *priv = i2c_get_clientdata(client);

	enable_irq(priv->dr_irq);

	if (device_may_wakeup(&client->dev)) {
#ifdef CONFIG_MACH_OLPC_XO_1_75
		mfp_write(priv->gpio_data_ready, priv->save_mfpr);
#endif
		disable_irq_wake(priv->dr_irq);
	}
	else {
		int ret = zforce_command(priv, ZF_Activate, NULL, 0, NULL, 0);
		if (ret) {
			printk(KERN_WARNING "%s: ZF_Activate failed, ret %d\n",
				__func__, ret);
		}

		ret = zforce_command(priv, ZF_TouchData, NULL, 0, NULL, -1);
		if (ret) {
			printk(KERN_WARNING "%s: ZF_TouchData failed, ret %d\n",
				__func__, ret);
		}

	}

	return 0;
}

static SIMPLE_DEV_PM_OPS(zforce_pm, zforce_suspend, zforce_resume);

#endif


static struct i2c_driver zforce_i2c_driver = {
	.driver = {
		.name = DRIVER_NAME,
		.owner = THIS_MODULE,
#ifdef CONFIG_PM
		.pm = &zforce_pm,
#endif
	},
	.probe = zforce_i2c_probe,
	.remove = __devexit_p(zforce_i2c_remove),
	.id_table = zforce_i2c_id,
};

static int
zforce_init(void)
{
	return i2c_add_driver(&zforce_i2c_driver);
}

static void
zforce_exit(void)
{
	i2c_del_driver(&zforce_i2c_driver);
}

module_init(zforce_init);
module_exit(zforce_exit);

MODULE_AUTHOR("Paul Fox <pgf@laptop.org>");
MODULE_DESCRIPTION("Neonode zForce IR touchscreen driver");
MODULE_LICENSE("GPL");

