/*
 * Neonode zForce IR touchscreen driver
 *
 *   Copyright (C) 2010 One Laptop per Child Foundation.
 *
 * Licensed under the GPL-2 or later.
 */

/*
 * the ZF_xxx identifiers are used in both the commands and their
 * corresponding responses.  all commands generate a single
 * corresponding response, except for ZF_TouchData.
 */

#define ZF_Deactivate 0x00
	// o: 0 bytes	    deactivates the touchscreen
	// i: 1 byte	    success/fail


#define ZF_Activate 0x01
	// o: 0 bytes	    activates the touchscreen
	// i: 1 byte	    success/fail


#define ZF_Resolution 0x02
	// o: 4 bytes	    sets the logical width and height
	//		    of the returned coordinates.
	// i: 1 byte	    success/fail
struct SizeInfo {
	u16 width;
	u16 height;
};


#define ZF_Configuration 0x03  
	// o: 4 bytes	    enables or disables any customized
	//		    features by setting corresponding flag.
	// i: 1 byte	    success/fail
struct ConfigurationInfo {
	u32 flags;
};
#define ZF_FLAG_DUALTOUCH  1


#define ZF_TouchData 0x04
	// o: 0 bytes	      requests coordinate updates.
	// i: 1 + (9 * count) bytes
	//	for each touch (as counted by the first byte), one
	//	info structure is appended to the report.  unlike
	//	all other commands, multiple responses should be
	//	expected, generated on touch detection and motion.

struct TouchInfo {
	u8 xlow;
	u8 xhigh;
	u8 ylow;
	u8 yhigh;
	u8 id_state;	// high 6 bits are id, low 2 bits are state
		// TouchData is sent whenever the touch changes: 
		// "id" identifies an actual touch and remains
		//     constant until that touch is released.
		// "state" represents the state of the touch.
		//     when the touch is first recognized, state is 0,
		//     when the touch moves, state is 1.  when the
		//     touch is finally removed, state becomes 2.
#define ZF_TOUCHSTATE_MASK      3
#define ZF_TOUCHSTATE_DOWN      0
#define ZF_TOUCHSTATE_MOVE      1
#define ZF_TOUCHSTATE_UP        2
#define ZF_TOUCHSTATE_INVALID   3


	u8 width;		// mm, range 0-255
	u8 height;		// mm, range 0-255
	u8 pressure;		// range 0-255, but always reported as 0.
	u8 probability;		// range 0-100
};

struct TouchResponse {
	u8 count;
	struct TouchInfo touch[0]; // actually 'count' elements
};


#define ZF_BootComplete 0x07
	// i: 1 byte	    success/fail
	// 	after a reboot, indicates that the controller is
	// 	up and running and ready to receive input.


#define ZF_ScanningFrequency 0x08
	// o: 6 bytes	    sets the scanning frequency for
	//		    idle and full scanning mode.
	// i: 1 byte	    success/fail
	// these numbers affect detection latency, and power
	// consumption.  changes can be made without deactivating
	// touch reports.  (the pen frequency would be used a on
	// a device with specific pen support, but isn't used on
	// the OLPC XO-4 screen.)
struct ScanningFrequencyInfo {
	u16 idleFrequency;
	u16 fullScanningFrequency;
	u16 penFrequency;
};


#define ZF_TouchAreaSizes 0x09
	// o: 4 bytes	    sets the maximum or minimum size of
	//		    object that generates touch.
	// i: 1 byte	    success/fail
	// size is expressed as a dimension in mm, and is a
	// minimum for _both_ x and y -- i.e., a 31mm x 5mm touch
	// won't trigger if the configured minimum is 20mm. 
	// changes can be made without deactivating touch
	// reports.
struct TouchAreaSizeInfo {
	u8 enabled;			// 0 = off, 1 = on
	u8 size;			// In millimeter
};

struct TouchAreaSizes {
	struct TouchAreaSizeInfo max;
	struct TouchAreaSizeInfo min;
};


#define ZF_LowSignals 0x0D   // for debug
	//  o: 1 byte	    requests low signal response for
	//		    the selected axis.	(X = 0, Y = 1)
	//  i: 2 bytes + numAxisSignals bits, packed in bytes
	//		    returns a bool for each signal indicating
	//		    if it’s too low for normal operation.
	//		    last byte may contain dummy bits.

struct LowSignalResponse { // in
	u8 axis;		// X = 0, Y = 1
	u8 numAxisSignals;
	u8 LowSignal[8];	// actually '((numAxisSignals+7) / 8)' bytes
};


#define ZF_FixedPulseStrength 0x0F   // for debug
	//  o: 2 bytes	    performs a complete axis scan with
	//		    the requested LED strength.
	//  i: 2 + numAxisSignals bytes
struct FixedPulseInfo { // out
	u8 axis;		// X = 0, Y = 1
	u8 strength;		//ASIC (1-64), Discrete(1-90)
};

struct FixedPulseStrengthResponse { // in
	u8 axis;		// X = 0, Y = 1
	u8 numAxisSignals;
	u8 PDSignal[252];	// actually 'numAxisSignals' elements
};


#define ZF_ForceCalibration 0x1A   // debug
	// o: 0 bytes	   forces a LED calibration.
	// i: 1 byte	    success/fail


#define ZF_LEDLevels 0x1C   // for debug
	// o: 1 byte	    requests LED and photodiode information
	//		    for the selected axis.  (X = 0, Y = 1)
	// i: 2+(3 * numAxisSignals)  bytes

struct LEDSignalInfo {
	u8 LEDStrength;  // low 4 bits: strength1, high 4 bits: strength2
	u8 PDSignal1;
	u8 PDSignal2;
};

struct LEDLevelResponse {
	u8 axis;		// X = 0, Y = 1
	u8 numAxisSignals;
	struct LEDSignalInfo levels[84]; // actually 'numAxisSignals' elements
};


#define ZF_ActiveLeds 0x1D
	// o: 4 bytes	    sets the X and Y LEDs to be used in scanning.
	//		    this lets you define the active touch area.
	//		    The range for X-axis is 0 to (NumberOfXLEDs - 1)
	//		    and for Y it’s 0 to (NumberOfYLEDs - 1).
	//		    Overrides default settings.
	// i: 1 byte	    success/fail

struct ActiveLEDsInfo {
	u8 x_Start;
	u8 x_End;
	u8 y_Start;
	u8 y_End;
};


#define ZF_StatusCommand 0x1E
	// o: 0 bytes	    requests current MCU settings.
	// i: 128 bytes	    response contains the current MCU settings.

struct SettingsResponse {
	u16 major;		// Firmware version
	u16 minor;		// Firmware version
	u16 build;		// Firmware version
	u16 revision;		// Firmware version
	u8 active_touchcount;	// bit 7: active, bit0-bit6: touchcount
	u16 scanningCounter;
	u16 preparedTouchPackages;
	u16 sentTouchPackages;
	u16 invalidTouchCounter;
	struct ConfigurationInfo configuration;
	struct ScanningFrequencyInfo frequencyData;
	struct SizeInfo resolution;
	struct SizeInfo physicalSize;	// Millimeter
	struct ActiveLEDsInfo activeLEDs;
	u8 maxStrength;
	struct TouchAreaSizeInfo maxSize;
	struct TouchAreaSizeInfo minSize;
	u8 RESERVED[0];  // pads to 128 bytes
} __attribute__ ((packed));



#define ZF_ForcedLEDLevels 0x20
	// o: 1 byte	    forces a calibration and requests LED and
	//		    photodiode information for the selected axis.
	//		    (X = 0, Y = 1)
	// i: 2+(3 * numAxisSignals)  bytes
	// results are the same as for ZF_LEDLevels, above


#define ZF_OpenShort 0x21
	// o: 1 byte	    requests the open/short data for both LED and
	//		    PD on the corresponding axis.  (X = 0, Y = 1)
	// i: (2 + (2 * NumberOfAxisPDs + 2 * NumberOfAxisLEDs + 7) / 8) bytes
	//		    Result is byte aligned so may contain dummy
	//		    bits at the end for fill out.
	//
struct OpenShortResponse {
	u8 axis;
	u8 numAxisSignals;
	u8 bits[128];   // max 'numAxisSignals' is 255.  255*4/8 = 128
	// bit Open_PD[NumberOfAxisPDs];
	// bit Short_PD[NumberOfAxisPDs];
	// bit Short_LED[NumberOfAxisLEDs];
	// bit Open_LED[NumberOfAxisLEDs];
};


#define ZF_Overrun 0x25
	// i: 0 byte	    sent when a receive overrun is detected.
	//		    All received characters in previous
	//		    transmission are flushed.


#define ZF_InvalidCommand 0xFE
	// i: 1 byte	    this is a response to an unknown command or an
	//		    invalid argument used in a command.	 axis and
	//		    port command parameters are validated.


#ifdef UNIMPLEMENTED  // not part of OLPC hardware
// unimp #define ZF_SetGPIOControl 0x23
	// o: 3 bytes	    sets the port to input/output with low/high state.
	// i: 1 byte	    success/fail
struct GPIOInfo {
	u8 port;		// 1-4 (P1OUT – P4OUT)
	u8 direction;		// Each bit corresponds to port bit
				// 1 = Output, 0 = Input
	u8 state;		// 1 = High, 0 = Low, Don’t care
				// if direction == 0.
};


// unimp #define ZF_GetGPIOControl 0x24
	// o: 1 byte	    requests the port current information.
	// i: 3 bytes	    This response gets the port status. Also sent
	//		    asynchronously when the state of a configured
	//		    input IO changes.

struct GPIOInfoResponse {
	u8 port;		// 1-4 (P1OUT – P4OUT)
	u8 direction;		//1 = Output, 0 = Input
	u8 state;		//1 = High, 0 = Low
};

// unimp #define ZF_RunProximity 0x26
	// o: 1 bytes	    starts/stops proximity scanning. No response is sent.
	// i: 3 bytes	    sent when the proximity sensor is covered or
	//		    uncovered and proximity has been enabled.
struct RunProximity {
	u8 run;			// 1 = run, 0 = stop
};

struct ProximityResponse {
	u8 iscovered_id;  // bit 7 is "iscovered", bits 0-6 are id
	u16 reserved;
} __attribute__ ((packed));

// unimp #define ZF_ProductIDRequest 0x29
	// o: 0
	// i: N product ID bytes, as specified by customer

#endif
